/* Program : gen_cp2txt.dll
   Description : Dumps currently playing song from Winamp to a file
   By : Lennart Johannesson
   Date : May 28th, 2005
   Language : C

   E-Mail: lentoman@lentoman.net or lennart.johannesson@gmx.net
   Webpage: www.lentoman.net
   
   History:  
   0.4 - Support for half-life (by calling output file .cfg)
   0.3 - Configuration of output filename now possible
   0.2 - Added filtering, outputpath still hardcoded
   0.1 - Simple dump of windowtitle to a static-hardcoded path

   To do:
   *Real Gui for config
   *Config for enabling/disabling plugin
   *Setup multiple paths: hl.cfg and regular.txt output
*/


#include <windows.h> 
#include <stdio.h>
#include <string.h>
#include "gen_cp2txt.h" 

//#define debug

int init(void); 
void config(void); 
void quit(void);
void config_write(void);
void config_read(void);

void removeAllOccurances(char *s, char c);
void writeCurrentSong(void);
void startTimer(void);

VOID CALLBACK SongChangeCheck(HWND hwnd,UINT uMsg,UINT idEvent,DWORD dwTime);

UINT_PTR g_pTimerID;

char outputOk = 0; // Will be set to 1 if output file is configured
char pluginEnabled = 1; // Plugin enabled or not?
char lastWrittenSongTitle[1024];
char currentSongTitle[1024] = "<Nothing>";
char configPath[MAX_PATH]; // gen_cp2txt.ini
char outputPath[MAX_PATH] = "";

winampGeneralPurposePlugin plugin = {
	GPPHDR_VER,
	"Currently Playing To Textfile v0.4 (gen_cp2txt.dll)", // Plug-in description 
	init,
	config, 
	quit,
}; 

void removeAllOccurances(char *s, char c)
{
	char *pFound = NULL;
	int i;
	
	while ((pFound = strchr( s, c )) != NULL){
	// (pFound-s) is the position at where we found the character
	    for ( i = (pFound - s ); i <= strlen(s); i++)
			s[i]=s[i+1];
	}
				
}
 
void writeCurrentSong()
{
	FILE *fOutput;
	int curPos=0,
		tempPos;
	char cleanSongTitle[1024] = "",
		*pTemp = NULL;

	if ( !(outputOk && pluginEnabled)) 
		return; // Not Configured and active, so we return...
	
	strcpy( lastWrittenSongTitle, currentSongTitle ); // Saved for comparing in timerfunction
	strcpy( cleanSongTitle, currentSongTitle ); // Our copy to be filtered

// First, we filter out bad characters that possibly can be exploited in mIRC ( % $ / and |)
	removeAllOccurances( cleanSongTitle, '%');
	removeAllOccurances( cleanSongTitle, '$');
	removeAllOccurances( cleanSongTitle, '/');
	removeAllOccurances( cleanSongTitle, '|');

/* Incase numbered list is turned on, we will filter out numbers here.
   NOTE: Might mess with possible short filenames or filenames with early dots
		in the name */
  
	if ( (pTemp = strchr( cleanSongTitle, '.' )) != NULL ){ // . (dot) found?
	
		if ((pTemp - cleanSongTitle) < 8) /* Dot is only valid if it is in any of the first 8 characters,
											allows up to 999999 mp3s */
			strcpy( cleanSongTitle, pTemp + 2); // Start copying title after dotspace ". "
	}
/* Lastly, let's remove the " - Winamp" string at the end of the name */

	if ( (pTemp = strstr ( cleanSongTitle, " - Winamp" )) != NULL)
		cleanSongTitle[pTemp-cleanSongTitle] = '\0'; // Terminating string earlier, ugly but works...
	
	if ( fOutput = fopen(outputPath, "w") ) {
		
		if( strstr(outputPath,".cfg") != NULL ) { // If .cfg extension
			fwrite("say \"Currently Listening To: " , 1 , strlen("say \"Currently Listening To: ") , fOutput);
			fwrite(cleanSongTitle , 1 , strlen(cleanSongTitle) , fOutput); 
			fwrite("\"" , 1 , strlen("\"") , fOutput); // add " at end
			fwrite("\n" , 1 , strlen("\n") , fOutput); // Newline
			fwrite("//" , 1 , strlen("//") , fOutput); // Make Comment
		}
// Clean "safe" title
		fwrite(cleanSongTitle , 1 , strlen(cleanSongTitle) , fOutput); 
		fwrite("\n" , 1 , strlen("\n") , fOutput); // Newline

// Nonclean title (Could send exploits to mIRC)
		if( strstr(outputPath,".cfg") != NULL ) // If HL.cfg
			fwrite("//" , 1 , strlen("//") , fOutput); // Make Comment
		fwrite(currentSongTitle , 1 , strlen(currentSongTitle) , fOutput);
		fwrite("\n" , 1 , strlen("\n") , fOutput); // Newline
		
		fclose(fOutput); // Close file

	}
}
 

void startTimer()
{
	g_pTimerID = SetTimer(plugin.hwndParent, 12321, 5000, (TIMERPROC)SongChangeCheck);
	
	if ( g_pTimerID == 0)
		MessageBox(plugin.hwndParent,"gen_cp2txt.dll could not aquire a timer and will not be active! :(","",MB_OK); 
}

VOID CALLBACK SongChangeCheck(HWND hwnd,UINT uMsg,UINT idEvent,DWORD dwTime)
{
	GetWindowText(plugin.hwndParent, currentSongTitle, sizeof(currentSongTitle));
	
	if ( !(strcmp(currentSongTitle, lastWrittenSongTitle) == 0) ) {
		writeCurrentSong();
#ifdef debug		 
		MessageBox(plugin.hwndParent,currentSongTitle,"",MB_OK); 
#endif
	}
}

int init() {
	char *pTemp;
	
	/* Create config filename */
	GetModuleFileName(plugin.hDllInstance,configPath,sizeof(configPath));
	
	if ( (pTemp = strstr ( configPath, ".dll" )) != NULL)
		strcpy( pTemp, ".ini"); // replace .dll with .ini
	
	config_read();
	
	startTimer();
	writeCurrentSong();

#ifdef debug	
	MessageBox(plugin.hwndParent,"Init","",MB_OK); 
#endif

	return 0;
} 

void config() { 
	OPENFILENAME ofn;
	char tempPath[MAX_PATH] = "";

	ZeroMemory(&ofn, sizeof(ofn));
	tempPath[0] = 0;

	ofn.lStructSize = sizeof(ofn);
	ofn.lpstrFilter = "Textfile (*.txt)\0*.txt\0Half-Life Configfile (*.cfg)\0*.cfg\0\0";
	ofn.lpstrFile = tempPath;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrDefExt = "txt";

	ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;

	if ( GetSaveFileName(&ofn) )
	{
		strcpy ( outputPath, tempPath);
		config_write();
		outputOk = 1; // Everything OK output may be written
		writeCurrentSong(); // Write current song to new file
	}
	  
// outputPath

#ifdef debug
	MessageBox(plugin.hwndParent,"Config","",MB_OK); 
#endif
} 

void quit() { 
	KillTimer( plugin.hwndParent, g_pTimerID);
	strcpy( currentSongTitle, "<Winamp Is Not Running>" );
	writeCurrentSong();
#ifdef debug
	MessageBox(0,"Quit","",MB_OK); 
#endif

} 

void config_read()
{
	FILE *fInput;
	
	// This will fail if not configed yet
	if (fInput = fopen(configPath, "r"))
	{
		if ( fgets(outputPath, MAX_PATH, fInput) ) // Read [gen_cp2txt.dll]
			if ( fseek( fInput, 11, SEEK_CUR ) == 0 ) // Don't read 'OutputFile='
				if ( fgets(outputPath, MAX_PATH, fInput) ) // Read actual path
					outputOk = 1; // Everything OK output may be written
			  
		fclose( fInput );
	}	
}

void config_write()
{
	FILE *fOutput;

	if ( fOutput = fopen(configPath, "w") ) {
		fwrite("[gen_cp2txt.dll]" , 1 , strlen("[gen_cp2txt.dll]") , fOutput); 
		fwrite("\nOutputFile=" , 1 , strlen("\nOutputFile=") , fOutput); // Newline
		fwrite(outputPath , 1 , strlen(outputPath) , fOutput);
		fclose(fOutput); 
	}
}

__declspec(dllexport) winampGeneralPurposePlugin * winampGetGeneralPurposePlugin() {
  return &plugin; 
}
